/*:ja
 * @plugindesc v1.19 メッセージの表示方法や機能をカスタマイズすることができます。
 * @author Yanfly Engine Plugins
 *
 * @param ---一般---
 * @default
 *
 * @param Default Rows
 * @parent ---一般---
 * @type number
 * @min 0
 * @desc メッセージボックスに表示されるデフォルトの行数を指定
 * Default: 4
 * @default 4
 *
 * @param Default Width
 * @parent ---一般---
 * @desc メッセージボックスの幅をピクセルで指定
 * Default: Graphics.boxWidth
 * @default Graphics.boxWidth
 *
 * @param Face Indent
 * @parent ---一般---
 * @desc 顔グラフィックを使っている時の、テキストインデント値を指定
 * Default: Window_Base._faceWidth + 24
 * @default Window_Base._faceWidth + 24
 *
 * @param Fast Forward Key
 * @parent ---一般---
 * @desc このキーを使うと早送りする事が出来ます。
 * @default pagedown
 *
 * @param Enable Fast Forward
 * @parent ---一般---
 * @type boolean
 * @on 有効
 * @off 無効
 * @desc デフォルトで早送りボタンを有効
 * 無効 - false   有効 - true
 * @default true
 *
 * @param Word Wrapping
 * @parent ---一般---
 * @type boolean
 * @on ON
 * @off OFF
 * @desc ワードラップ機能の ON/OFF を設定
 * OFF - false   ON - true
 * @default false
 *
 * @param Description Wrap
 * @parent ---一般---
 * @type boolean
 * @on ON
 * @off OFF
 * @desc 詳細の欄でのワードラップ機能の ON/OFF を設定
 * OFF - false   ON - true
 * @default false
 *
 * @param Word Wrap Space
 * @parent ---一般---
 * @type boolean
 * @on 有効
 * @off 無効
 * @desc 手動改行を有効
 * 無効 - false   有効 - true
 * @default false
 *
 * @param Tight Wrap
 * @parent ---一般---
 * @type boolean
 * @on YES
 * @off NO
 * @desc 顔画像使用時メッセージをタイトに折り返すか設定。 NO - false   YES - true
 * @default false
 *
 * @param ---フォント---
 * @default
 *
 * @param Font Name
 * @parent ---フォント---
 * @desc メッセージウィンドウに使われるデフォルトのフォントを設定
 * Default: GameFont
 * @default GameFont
 *
 * @param Font Name CH
 * @parent ---フォント---
 * @desc 中国語のRPGに使われるデフォルトフォントを設定
 * Default: SimHei, Heiti TC, sans-serif
 * @default SimHei, Heiti TC, sans-serif
 *
 * @param Font Name KR
 * @parent ---フォント---
 * @desc 韓国語のRPGに使われるデフォルトフォントを設定
 * Default: Dotum, AppleGothic, sans-serif
 * @default Dotum, AppleGothic, sans-serif
 *
 * @param Font Size
 * @parent ---フォント---
 * @type number
 * @min 1
 * @desc メッセージウィンドウに使われるデフォルトのフォントサイズを指定
 * Default: 28
 * @default 28
 *
 * @param Font Size Change
 * @parent ---フォント---
 * @type number
 * @min 1
 * @desc { と } が使われた際、このフォントサイズを適用
 * Default: 12
 * @default 12
 *
 * @param Font Changed Max
 * @parent ---フォント---
 * @type number
 * @min 1
 * @desc { によって生じるフォントの最大サイズを設定
 * Default: 96
 * @default 96
 *
 * @param Font Changed Min
 * @parent ---フォント---
 * @type number
 * @min 1
 * @desc { によって生じるフォントの最小サイズを設定
 * Default: 12
 * @default 12
 *
 * @param Font Outline
 * @parent ---フォント---
 * @type number
 * @min 0
 * @desc アウトラインフォントに使われるデフォルトの横幅を設定
 * Default: 4
 * @default 4
 *
 * @param Maintain Font
 * @parent ---フォント---
 * @type boolean
 * @on YES
 * @off NO
 * @desc フォント名またはフォントサイズを変更した時に、次のメッセージを続けるか設定   NO - false   YES - true
 * @default false
 *
 * @param ---ネームボックス---
 * @default
 *
 * @param Name Box Buffer X
 * @parent ---ネームボックス---
 * @type number
 * @desc ネームボックスのバッファを指定します  (x軸)
 * @default -28
 *
 * @param Name Box Buffer Y
 * @parent ---ネームボックス---
 * @type number
 * @desc ネームボックスのバッファを指定します  (y軸)
 * @default 0
 *
 * @param Name Box Padding
 * @parent ---ネームボックス---
 * @desc ネームボックスのパディング値を指定
 * @default this.standardPadding() * 4
 *
 * @param Name Box Color
 * @parent ---ネームボックス---
 * @type number
 * @min 0
 * @max 31
 * @desc ネームボックスに使われるテキスト色を指定
 * @default 0
 *
 * @param Name Box Clear
 * @parent ---ネームボックス---
 * @type boolean
 * @on YES
 * @off NO
 * @desc ネームボックスを空欄にしますか？ NO - false     YES - true
 * @default false
 *
 * @param Name Box Added Text
 * @parent ---ネームボックス---
 * @desc ネームボックスが使われる際には必ずこのテキストが追加されます。自動的にカラー設定をしたい場合などに使われます。
 * @default c[6]
 *
 * @param Name Box Auto Close
 * @parent ---ネームボックス---
 * @type boolean
 * @on YES
 * @off NO
 * @desc ネームボックスに別の名前が表示される時に、メッセージウィンドウを閉じるか設定。   YES - true     NO - false
 * @default false
 *
 * @help
 * 翻訳:ムノクラ
 * https://munokura.tk/
 * https://twitter.com/munokura/
 *
 * ============================================================================
 * Introduction
 * ============================================================================
 *
 * RPGツクールMVではメッセージシステムが強化されましたが、このプラグインを使えば
 * 更にアイテム・武器・防具・衣装の名前や、アイコンのテキストコードの
 * 変換を容易に行なうことができます。
 * 
 * またこのスクリプトでは、ゲーム中にメッセージウィンドウのサイズを最適化したり
 * セパレートフォントや、テキストの早送り機能を提供します。
 *
 * ============================================================================
 * Word Wrapping
 * ============================================================================
 *
 * ワードラップ機能は、メッセージシステム上で使うことができます。
 * プラグインコマンドを用いて、有効/無効の切り替えを行ってください。
 * ワードラップを使うことで、メッセージウィンドウ外へはみ出た文章を、
 * 自動的に次の行に折り返します。この際、エディターの改行入力は無効化され、
 * プラグインによる改行が優先されます。
 * 
 * <br>もしくは<line break>というテキストコードを使って、改行を行います。
 * 新しい行を始めたい部分の前、もしくは後にこのコードを入力してください。
 *
 * ワードラップ機能は、メッセージウィンドウ向けの機能ですが、
 * アイテム詳細など、それ以外の部分にもこの機能を用いたい場合、
 * そのテキストの先頭に<WordWrap>と挿入すれば、利用することができます。
 *
 * ============================================================================
 * Text Codes
 * ============================================================================
 *
 * メッセージに特定のテキストコードを使用し、
 * 次のコードに置き換えることができます。
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 * Text Code   Function
 *   V[n]       n 番目の変数の値と置き換えられます。
 *   N[n]       n 番目のアクター名と置き換えられます。
 *   P[n]       n 番目のパーティメンバー名と置き換えられます。
 *   G          通貨単位と置き換えられます。
 *   C[n]       続くテキストを、n 番目のカラーで表示します。
 *   I[n]       n 番目のアイコンを表示します。
 *   {          テキストサイズを一段階大きくします。
 *   }          テキストサイズを一段階小さくします。
 *   \          のキャラクターと置き換えられます。
 *   $          所持金ウィンドウを開きます。
 *   .          1/4 秒の間をあけます。
 *   |          1 秒の間をあけます。
 *   !          ボタンインプットを待ちます。
 *   >          同じラインの残りのテキストを一気に表示します。
 *   <          上記のエフェクトをキャンセルします。
 *   ^          テキスト表示後にインプットを待たなくなります。
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  Wait:       Effect:
 *    w[x]     -  x フレーム分の間をあけます (60フレーム = 1秒) 
 *                 メッセージウィンドウ限定の機能です。 
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  NameWindow: Effect:
 *    n<x>     - 左揃えで x の文字列でネームボックスを作成します。 *注
 *    nc<x>    - 中央揃えで x の文字列でネームボックスを作成します。 *注
 *    nr<x>    - 右揃えで x の文字列でネームボックスを作成します。 *注
 *
 *              *注:メッセージウィンドウのみに有効
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  Line Break  Effect:
 *    <br>      - ワードラップモードで改行を行います。
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  Position:   Effect:
 *    px[x]    - テキストのx方向の位置を指定
 *    py[x]    - テキストのy方向の位置を指定
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  Outline:    Effect:
 *   oc[x]    - アウトラインの色を x にします。
 *   ow[x]    - アウトラインの幅を x にします。
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  Font:       Effect:
 *    fr       - 全てのフォント変更をリセットします。
 *    fs[x]    - フォントサイズを x に変更します。
 *    fn<x>    - フォント名を x に変更します。
 *    fb       - ボールド設定を切り替えます。
 *    fi       - イタリック設定を切り替えます。
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  Actor:      Effect:
 *    af[x]    - アクター x の顔を表示します。(注)
 *    ac[x]    - アクターの職業名を表示します。
 *    an[x]    - アクターのニックネームを表示します。
 *
 *              注:メッセージウィンドウのみで有効
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  Party:      Effect:
 *    pf[x]    - パーティメンバー x の顔を表示します。*注
 *    pc[x]    - パーティメンバー x の職業名を表示します。
 *    pn[x]    - パーティメンバー x のニックネームを表示します。
 *
 *              *注:メッセージウィンドウのみで有効
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  Names:      Effect:
 *    nc[x]    - 職業 x の名前を表示します。
 *    ni[x]    - アイテム x の名前を表示します。
 *    nw[x]    - 武器 x の名前を表示します。
 *    na[x]    - 防具 x の名前を表示します。
 *    ns[x]    - スキル x の名前を表示します。
 *    nt[x]    - ステート x の名前を表示します。
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *  Icon Names: Effect:
 *    nc[x]    - 職業 x の名前を表示します。
 *    ni[x]    - アイテム x の名前を表示します。
 *    nw[x]    - 武器 x の名前を表示します。
 *    na[x]    - 防具 x の名前を表示します。
 *    ns[x]    - スキル x の名前を表示します。
 *    nt[x]    - ステート x の名前を表示します。
 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 * これらはスクリプトと共に加えられたテキストコードです。これらのコードは、
 * メッセージウィンドウにのみ働くものであることに気を付けてください。
 * またそれ以外にも、ヘルプの詳細、アクタープロフィールなどにも有効です。
 *
 * ============================================================================
 * Plugin Commands
 * ============================================================================
 *
 * イベントエディタを通して下記のプラグインコマンドを用いることで、メッセージ
 * システムの様々な要素を変更することができます。
 *
 * プラグインコマンド
 *   MessageRows 6          
 *   - 表示されるメッセージの列を6にします。
 *     もし［文章の表示］を継続的に使っている場合、
 *     行の上限に到達するまで、
 *     あとに続く文章を表示し続けます。
 *     それ以降の文章については、
 *     重複回避のためにカットされます。
 *
 *   MessageWidth 400       
 *   - メッセージウィンドウの幅を400ピクセルに変更します。
 *     右側に寄りすぎている文字は、
 *     適宜カットされ調整が行われます。
 *
 *   EnableWordWrap         
 *   - ワードラップを有効にします。
 *     文がウィンドウサイズを超えてしまった時に、自動的に次の行に進みます。
 *     改行する際は 'br'を挿入してください。
 *
 *   DisableWordWrap        
 *   - ワードラップを無効にします。
 *     エディタ内で新しい文が始まった地点で、自動的に改行されます。
 *
 *   EnableFastForward
 *   - メッセージ内の早送りキーを有効にします。
 *
 *   DisableFastForward
 *   - メッセージ内の早送りキーを無効にします。
 *
 * ============================================================================
 * Changelog
 * ============================================================================
 *
 * Version 1.19:
 * - Updated for RPG Maker MV version 1.5.0.
 *
 * Version 1.18:
 * - Added new plugin parameters: 'Font Name CH' and 'Font Name KR'.
 *
 * Version 1.17:
 * - Compatibility update with Message Macros for 'Name Box Auto Close' option.
 *
 * Version 1.16:
 * - Added 'Tight Wrap' plugin parameter as a word wrap option to make the
 * word wrap tighter when using faces.
 *
 * Version 1.15:
 * - Added a failsafe where if the name box window would be off the screen, it
 * will automatically reposition itself to under the main message window.
 *
 * Version 1.14:
 * - Added 'Name Box Close' plugin parameter. If this is enabled, the message
 * window will check for the Name Window speaker each time a follow up message
 * occurs. If the name in the currently Name Window matches the name in the
 * following Name Window, the message window will remain open. If it doesn't,
 * the Name Window will close and reopen to indicate a new speaker.
 *
 * Version 1.13:
 * - Added 'Maintain Font' plugin parameter under the Font category. This will
 * allow you to use text codes fn<x> and fs[x] to permanently change the font
 * of your messages until you use it again. fr will reset them to the plugin's
 * default parameter settings.
 *
 * Version 1.12:
 * - 'Word Wrap Space' parameter no longer leaves a space at the beginning of
 * each message.
 *
 * Version 1.11:
 * - Added 'Font Outline' parameter for the plugin parameters. This adjusts the
 * font outline width used by default for only message fonts.
 *
 * Version 1.10:
 * - Updated the Message Row system for Extended Message Pack 1's Autosizing
 * feature to work with extended heights.
 *
 * Version 1.09:
 * - Replaced 'Fast Forward' parameter with the 'Fast Forward Key' parameter
 * and 'Enable Fast Forward' parameter. Two new Plugin Commands are added. They
 * are 'EnableFastForward' and 'DisableFastForward' for control over when fast
 * forwarding is allowed as to not cause timed cutscenes to desynch.
 *
 * Version 1.08:
 * - Fixed a bug regarding Input Number positioning when the Message Window's
 * position was middle.
 *
 * Version 1.07:
 * - Added 'Word Wrap Space' for word wrap users. This parameter will leave a
 * space behind for those who want a space left behind.
 *
 * Version 1.06:
 * - Fixed a bug that would cause masking problems with mobile devices.
 *
 * Version 1.05:
 * - Fixed a bug that would cause the namebox window to appear distorted.
 *
 * Version 1.04:
 * - Fixed a bug that captured too many text codes with the namebox window.
 * - Timed Name Window's closing speed with main window's closing speed.
 *
 * Verison 1.03:
 * - Fixed a bug with textcodes that messed up wordwrapping.
 * - Fixed a bug with font reset, italic, and bold textcodes.
 *
 * Version 1.02:
 * - Namebox Window's overlap feature that's in every MV window is now disabled
 * to allow for overlapping with main message window.
 * - Updated window positioning for Branch Choices, Number Input, and Item
 * Selection windows.
 *
 * Version 1.01:
 * - Added 'Description Wrap' into the parameters to allow for all item
 * descriptions to be automatically processed with word wrapping.
 *
 * Version 1.00:
 * - Finished plugin!
 */